@@ -6,12 +6,13 @@ The authors of original MinAtar implementation are:
 The original MinAtar implementation is distributed under GNU General Public License v3.0
     * https://github.com/kenjyoung/MinAtar/blob/master/License.txt
 """
-from typing import Tuple
+from typing import Literal, Optional
 
 import jax
 import jax.lax as lax
 from jax import numpy as jnp
 
+import pgx.core as core
 from pgx._flax.struct import dataclass
 
 RAMP_INTERVAL: jnp.ndarray = jnp.int32(100)
@@ -32,45 +33,103 @@ FALSE: jnp.ndarray = jnp.bool_(False)
 
 
 @dataclass
-class State:
+class State(core.State):
+    current_player: jnp.ndarray = jnp.int8(0)
+    observation: jnp.ndarray = jnp.zeros((10, 10, 10), dtype=jnp.bool_)
+    reward: jnp.ndarray = jnp.zeros(1, dtype=jnp.float32)  # (1,)
+    terminated: jnp.ndarray = FALSE
+    truncated: jnp.ndarray = FALSE
+    legal_action_mask: jnp.ndarray = jnp.ones(6, dtype=jnp.bool_)
+    _rng_key: jax.random.KeyArray = jax.random.PRNGKey(0)
+    _step_count: jnp.ndarray = jnp.int32(0)
+    # --- MinAtar Seaquest specific ---
     oxygen: jnp.ndarray = MAX_OXYGEN
     diver_count: jnp.ndarray = ZERO
     sub_x: jnp.ndarray = jnp.int32(5)
     sub_y: jnp.ndarray = ZERO
     sub_or: jnp.ndarray = FALSE
-    f_bullets: jnp.ndarray = -jnp.ones(
-        (5, 3), dtype=jnp.int32
-    )  # <= 2  TODO: confirm
+    f_bullets: jnp.ndarray = -jnp.ones((5, 3), dtype=jnp.int32)
     e_bullets: jnp.ndarray = -jnp.ones(
         (25, 3), dtype=jnp.int32
-    )  # <= 1 per each sub  TODO: confirm
-    e_fish: jnp.ndarray = -jnp.ones(
-        (25, 4), dtype=jnp.int32
-    )  # <= 19  TODO: confirm
-    e_subs: jnp.ndarray = -jnp.ones(
-        (25, 5), dtype=jnp.int32
-    )  # <= 19  TODO: confirm
-    divers: jnp.ndarray = -jnp.ones(
-        (5, 4), dtype=jnp.int32
-    )  # <= 2  TODO: confirm
+    )  # <= 1 per each sub
+    e_fish: jnp.ndarray = -jnp.ones((25, 4), dtype=jnp.int32)  # <= 19
+    e_subs: jnp.ndarray = -jnp.ones((25, 5), dtype=jnp.int32)  # <= 19
+    divers: jnp.ndarray = -jnp.ones((5, 4), dtype=jnp.int32)  # <= 2
     e_spawn_speed: jnp.ndarray = INIT_SPAWN_SPEED
     e_spawn_timer: jnp.ndarray = INIT_SPAWN_SPEED
     d_spawn_timer: jnp.ndarray = DIVER_SPAWN_SPEED
     move_speed: jnp.ndarray = INIT_MOVE_INTERVAL
-    ramp_index: jnp.ndarray = ZERO  # TODO: require int32?
+    ramp_index: jnp.ndarray = ZERO
     shot_timer: jnp.ndarray = ZERO
     surface: jnp.ndarray = TRUE
     terminal: jnp.ndarray = FALSE
     last_action: jnp.ndarray = ZERO
 
+    def _repr_html_(self) -> str:
+        from pgx.minatar.utils import visualize_minatar
 
-def step(
+        return visualize_minatar(self)
+
+    def save_svg(
+        self,
+        filename,
+        *,
+        color_theme: Optional[Literal["light", "dark"]] = None,
+        scale: Optional[float] = None,
+    ) -> None:
+        from pgx.minatar.utils import visualize_minatar
+
+        visualize_minatar(self, filename)
+
+
+class MinAtarSeaquest(core.Env):
+    def __init__(
+        self,
+        *,
+        minatar_version: Literal["v0", "v1"] = "v1",
+        sticky_action_prob: float = 0.1,
+    ):
+        super().__init__()
+        self.minatar_version: Literal["v0", "v1"] = minatar_version
+        self.sticky_action_prob: float = sticky_action_prob
+
+    def _init(self, key: jax.random.KeyArray) -> State:
+        return _init_det()
+
+    def _step(self, state: core.State, action) -> State:
+        assert isinstance(state, State)
+        state = _step(
+            state, action, sticky_action_prob=self.sticky_action_prob
+        )
+        return state.replace(terminated=state.terminal)  # type: ignore
+
+    def _observe(
+        self, state: core.State, player_id: jnp.ndarray
+    ) -> jnp.ndarray:
+        assert isinstance(state, State)
+        return _observe(state)
+
+    @property
+    def name(self) -> str:
+        return "MinAtar/Seaquest"
+
+    @property
+    def version(self) -> str:
+        return "alpha"
+
+    @property
+    def num_players(self):
+        return 1
+
+
+def _step(
     state: State,
     action: jnp.ndarray,
-    rng: jnp.ndarray,
-    sticky_action_prob: jnp.ndarray,
-) -> Tuple[State, jnp.ndarray, jnp.ndarray]:
-    rngs = jax.random.split(rng, 6)
+    sticky_action_prob,
+):
+    key, subkey = jax.random.split(state._rng_key)
+    state = state.replace(_rng_key=key)  # type: ignore
+    rngs = jax.random.split(subkey, 6)
     action = jnp.int32(action)
     # sticky action
     action = jax.lax.cond(
@@ -101,11 +160,7 @@ def _step_det(
 ):
     return lax.cond(
         state.terminal,
-        lambda: (
-            state.replace(last_action=action),  # type: ignore
-            jnp.int32(0),
-            state.terminal,
-        ),
+        lambda: state.replace(last_action=action, reward=jnp.zeros_like(state.reward)),  # type: ignore
         lambda: _step_det_at_non_terminal(
             state, action, enemy_lr, is_sub, enemy_y, diver_lr, diver_y
         ),
@@ -142,7 +197,7 @@ def _step_det_at_non_terminal(
     surface = state.surface
     terminal = state.terminal
 
-    r = jnp.int32(0)
+    r = jnp.float32(0)
 
     # Spawn enemy if timer is up
     e_subs, e_fish = lax.cond(
@@ -227,7 +282,7 @@ def _step_det_at_non_terminal(
     )
     r += _r
 
-    state = State(
+    state = state.replace(  # type: ignore
         oxygen=oxygen,
         diver_count=diver_count,
         sub_x=sub_x,
@@ -247,8 +302,9 @@ def _step_det_at_non_terminal(
         surface=surface,
         terminal=terminal,
         last_action=action,
-    )  # type: ignore
-    return state, r, terminal
+        reward=r[jnp.newaxis],
+    )
+    return state
 
 
 def find_ix(arr):
@@ -415,38 +471,52 @@ def _update_enemy_subs(
             lambda: (_f_bullets, _e_subs, FALSE),
         )
         _r += removed
-        return _f_bullets, _e_subs, _terminal, _r
+        return (removed | is_out), _f_bullets, _e_subs, _terminal, _r
 
     def _update_each_filled(j, x):
         _f_bullets, _e_subs, _e_bullets, _terminal, _r = x
         _terminal |= _is_hit(_e_subs[j], sub_x, sub_y)
-        _f_bullets, _e_subs, _terminal, _r = lax.cond(
+        removed, _f_bullets, _e_subs, _terminal, _r = lax.cond(
             _e_subs[j, 3] == 0,
             lambda: _update_sub(j, _f_bullets, _e_subs, _terminal, _r),
-            lambda: (_f_bullets, _e_subs.at[j, 3].add(-1), _terminal, _r),
+            lambda: (
+                FALSE,
+                _f_bullets,
+                _e_subs.at[j, 3].add(-1),
+                _terminal,
+                _r,
+            ),
         )
         timer_zero = _e_subs[j, 4] == 0
         _e_subs = lax.cond(
-            timer_zero,
-            lambda: _e_subs.at[j, 4].set(ENEMY_SHOT_INTERVAL),
-            lambda: _e_subs.at[j, 4].add(-1),
+            removed,
+            lambda: _e_subs,
+            lambda: lax.cond(
+                timer_zero,
+                lambda: _e_subs.at[j, 4].set(ENEMY_SHOT_INTERVAL),
+                lambda: _e_subs.at[j, 4].add(-1),
+            ),
         )
         _e_bullets = lax.cond(
-            timer_zero,
-            lambda: _e_bullets.at[find_ix(_e_bullets)].set(
-                jnp.int32(
-                    [
-                        lax.cond(
-                            _e_subs[j, 2],
-                            lambda: _e_subs[j, 0],
-                            lambda: _e_subs[j, 0],
-                        ),
-                        _e_subs[j, 1],
-                        _e_subs[j, 2],
-                    ]
-                )
-            ),
+            removed,
             lambda: _e_bullets,
+            lambda: lax.cond(
+                timer_zero,
+                lambda: _e_bullets.at[find_ix(_e_bullets)].set(
+                    jnp.int32(
+                        [
+                            lax.cond(
+                                _e_subs[j, 2],
+                                lambda: _e_subs[j, 0],
+                                lambda: _e_subs[j, 0],
+                            ),
+                            _e_subs[j, 1],
+                            _e_subs[j, 2],
+                        ]
+                    )
+                ),
+                lambda: _e_bullets,
+            ),
         )
         return _f_bullets, _e_subs, _e_bullets, _terminal, _r
 
@@ -665,17 +735,13 @@ def _spawn_diver(divers, diver_lr, diver_y):
     return divers
 
 
-def init(rng: jnp.ndarray) -> State:
-    return _init_det()
-
-
-def observe(state: State) -> jnp.ndarray:
+def _observe(state: State) -> jnp.ndarray:
     obs = jnp.zeros((10, 10, 10), dtype=jnp.bool_)
-    obs = obs.at[state.sub_y, state.sub_x, 0].set(1)
+    obs = obs.at[state.sub_y, state.sub_x, 0].set(TRUE)
     back_x = lax.cond(
         state.sub_or, lambda: state.sub_x - 1, lambda: state.sub_x + 1
     )
-    obs = obs.at[state.sub_y, back_x, 1].set(1)
+    obs = obs.at[state.sub_y, back_x, 1].set(TRUE)
     oxygen_guage = state.oxygen * 10 // MAX_OXYGEN
     # hotfix to align to original minatar
     oxygen_guage = lax.cond(
@@ -684,13 +750,13 @@ def observe(state: State) -> jnp.ndarray:
     obs = lax.fori_loop(
         jnp.int32(0),
         oxygen_guage,
-        lambda i, _obs: _obs.at[9, i, 7].set(1),
+        lambda i, _obs: _obs.at[9, i, 7].set(TRUE),
         obs,
     )
     obs = lax.fori_loop(
         9 - state.diver_count,
         jnp.int32(9),
-        lambda i, _obs: _obs.at[9, i, 8].set(1),
+        lambda i, _obs: _obs.at[9, i, 8].set(TRUE),
         obs,
     )
     obs = lax.fori_loop(
@@ -700,7 +766,7 @@ def observe(state: State) -> jnp.ndarray:
             state.f_bullets[i][0] >= 0,
             lambda: _obs.at[
                 state.f_bullets[i][1], state.f_bullets[i][0], 2
-            ].set(1),
+            ].set(TRUE),
             lambda: _obs,
         ),
         obs,
@@ -712,18 +778,18 @@ def observe(state: State) -> jnp.ndarray:
             state.e_bullets[i][0] >= 0,
             lambda: _obs.at[
                 state.e_bullets[i][1], state.e_bullets[i][0], 4
-            ].set(1),
+            ].set(TRUE),
             lambda: _obs,
         ),
         obs,
     )
 
     def set_e_fish(_obs, fish):
-        _obs = _obs.at[fish[1], fish[0], 5].set(1)
+        _obs = _obs.at[fish[1], fish[0], 5].set(TRUE)
         back_x = fish[0] + jnp.array([1, -1])[fish[2]]
         _obs = lax.cond(
             (0 <= back_x) & (back_x <= 9),
-            lambda: _obs.at[fish[1], back_x, 3].set(1),
+            lambda: _obs.at[fish[1], back_x, 3].set(TRUE),
             lambda: _obs,
         )
         return _obs
@@ -740,11 +806,11 @@ def observe(state: State) -> jnp.ndarray:
     )
 
     def set_e_subs(_obs, sub):
-        _obs = _obs.at[sub[1], sub[0], 6].set(1)
+        _obs = _obs.at[sub[1], sub[0], 6].set(TRUE)
         back_x = sub[0] + jnp.array([1, -1], dtype=jnp.int32)[sub[2]]
         _obs = lax.cond(
             (0 <= back_x) & (back_x <= 9),
-            lambda: _obs.at[sub[1], back_x, 3].set(1),
+            lambda: _obs.at[sub[1], back_x, 3].set(TRUE),
             lambda: _obs,
         )
         return _obs
@@ -761,11 +827,11 @@ def observe(state: State) -> jnp.ndarray:
     )
 
     def set_divers(_obs, diver):
-        _obs = _obs.at[diver[1], diver[0], 9].set(1)
+        _obs = _obs.at[diver[1], diver[0], 9].set(TRUE)
         back_x = diver[0] + jnp.array([1, -1], dtype=jnp.int32)[diver[2]]
         _obs = lax.cond(
             (back_x >= 0) & (back_x <= 9),
-            lambda: _obs.at[diver[1], back_x, 3].set(1),
+            lambda: _obs.at[diver[1], back_x, 3].set(TRUE),
             lambda: _obs,
         )
         return _obs
