@@ -133,3 +133,8 @@ def test_observe():
          [False, True,  False, False, False, False, False]]
     )).all()
     assert obs[:, :, 1].sum() == 0
+
+def test_api():
+    import pgx
+    env = pgx.make("connect_four")
+    pgx.api_test(env, 10)
